import logging

logger = logging.getLogger(__name__)

class Base(object):
    """
        The Base class for Endpoint Client, which contains 2 wrapper class for
        handling endpoint response.
    """

    def _response_handler(self, cls, response):
        """ _response_handler function

            Convert response json to a Veem Object
        """
        if response and isinstance(response, dict):
            return cls(**response).convert
        logger.error(response)

    def _list_response_handler(self, cls, content_cls, response):
        """ _list_response_handler function

            Convert pagination list to a list of Veem Objects
        """
        if response:
            if isinstance(response, dict):
                return cls(**response).convertContent(content_cls)
            if isinstance(response, list):
                return cls(content=response).convertContent(content_cls)
        logger.error(response)




from veem.models.base import Base
from veem.models.address import Address

from veem.utils import deseralize
from veem.constants import ApprovalStatus

class BankAccount(Base):
    def __init__(self,
                 status=None,
                 email=None,
                 firstName=None,
                 lastName=None,
                 middleName=None,
                 **kwargs):

        self._validate_currency_code(currencyCode)
        self._validate_country_code(isoCountryCode)
        self._validate_constants(ApprovalStatus, status)

        self.routingNumber = routingNumber
        self.bankName = bankName
        self.bankAccountNumber = bankAccountNumber
        self.currencyCode = currencyCode
        self.isoCountryCode = isoCountryCode
        self.iban = iban
        self.swiftBic = swiftBic
        self.beneficiaryName = beneficiaryName
        self.bsbBankCode = bsbBankCode
        self.branchCode = branchCode
        self.transitCode = transitCode
        self.bankInstitutionNumber = bankInstitutionNumber
        self.bankIfscBranchCode = bankIfscBranchCode
        self.sortCode = sortCode
        self.bankCode = bankCode
        self.clabe = clabe
        self.bankCnaps = bankCnaps
        self.bankAddress = deseralize(Address, bankAddress)



from veem.client.responses.page import PageResponse
from veem.client.responses.payment import PaymentResponse

from veem.client.base import Base
from veem.utils.rest import VeemRestApi

class PaymentClient(Base):

    def __init__(self, config, **kwargs):
        self.config = config
        self.context = config.context
        self.client = VeemRestApi(self.config.url,
                                  self.context.session,
                                  dict(create=('post', ''),
                                       send=('post', '/{paymentId}/approve'),
                                       cancel=('post', '/{paymentId}/cancel'),
                                       get=('get', '/{paymentId}'),
                                       list=('get', '')))

    def create(self, request):
        """
            Create a Payment

            @param request: a PaymentRequest
            @return Payment that you just requested
            @throws VeemException If the provided PaymentRequest is invalid, or
                                  if creation fails.
        """
        return self._response_handler(PaymentResponse,
                                 self.client.create(
                                            access_token=self.context.token,
                                            api_route='payments',
                                            **(request.json))
                                     )

    def send(self, paymentId):
        """
            Send a specific payment by id

            @param request: payment id
            @return Payment that you just requested
            @throws VeemException If the provided paymentId is invalid, or
                                  if sending fails.
        """
        return self._response_handler(PaymentResponse,
                                 self.client.send(
                                        uri_params=dict(paymentId=paymentId),
                                        access_token=self.context.token,
                                        api_route='payments')
                                    )

    def cancel(self, paymentId):
        """
            Cancel a specific payment by id

            @param request: payment id
            @return Payment that you just requested
            @throws VeemException If the provided paymentId is invalid, or
                                  if cancelling fails.
        """
        return self._response_handler(PaymentResponse,
                                 self.client.cancel(
                                        uri_params=dict(paymentId=paymentId),
                                        access_token=self.context.token,
                                        api_route='payments')
                                    )

    def get(self, paymentId):
        """
            Get a specific payment by id

            @param request: payment id
            @return Payment that you just requested
            @throws VeemException If the provided paymentId is invalid, or
                                  if retriving fails.
        """
        return self._response_handler(PaymentResponse,
                                 self.client.get(
                                        uri_params=dict(paymentId=paymentId),
                                        access_token=self.context.token,
                                        api_route='payments')
                                    )

    def list(self, parameters={}):
        """
            Get all payments in pagination format

            @param request: a PaymentListParameters or customize dictionary with
                            listing criteria
            @return paginated Payments for your account
        """
        query = {}
        if parameters:
            query = getattr(parameters, 'json', {})
            query['sortParameters'] = ['%s:%s'%(
                    k, v) for field in query.get('sortParameters', {}).items()]
            query['paymentIds'] = ','.join(query.get('paymentIds', [])) or None
            query['status'] = ','.join(query.get('status', [])) or None
            query['sort'] = ','.join(query.pop('sortParameters', [])) or None
            query = {k:v for k,v in query.items() if v is not None}
        return self._list_response_handler(PageResponse,
                                 PaymentResponse,
                                 self.client.list(
                                                request_params=query,
                                                access_token=self.context.token,
                                                api_route='payments')
                                        )




import requests

class VeemContext(object):

    def __init__(self,
                 client_id=None, client_secret=None,
                 authorizationCode=None, redirectUrl=None,
                 **kwargs):

        self.clientId = client_id
        self.clientSecret = client_secret
        self.authorizationCode = authorizationCode
        self.redirectUrl = redirectUrl
        self.session = requests.Session()
        self.token = None




from .amount import AmountRequest
from .account import AccountRequest
from .attachment import AttachmentRequest

from veem.models.base import Base

from veem.utils import deseralize

class PaymentRequest(Base):
    def __init__(self,
                 batchItemId=None,
                 exchangeRateQuoteId=None,
                 payee=None,
                 amount=None,
                 notes=None,
                 externalInvoiceRefId=None,
                 ccEmails=[],
                 purposeOfPayment=None,
                 attachments=[],
                 approveAutomatically=False,
                 **kwargs):

        self.batchItemId = batchItemId
        self.exchangeRateQuoteId = exchangeRateQuoteId
        self.payee = deseralize(AccountRequest, payee).json
        self.amount = deseralize(AmountRequest, amount).json
        self.notes = notes
        self.externalInvoiceRefId = externalInvoiceRefId
        self.ccEmails = ccEmails
        self.purposeOfPayment = purposeOfPayment
        self.attachments = [deseralize(AttachmentRequest,
                                       attach).json for attach in attachments]
        self.approveAutomatically = approveAutomatically




from veem.models.base import Base
from veem.models.purpose_of_payment import PurposeOfPayment

from veem.utils import deseralize

class CountryInfo(Base):
    def __init__(self,
                 country=None,
                 sendingCurrencies=[],
                 receivingCurrencies=[],
                 purposeOfPaymentRequired=None,
                 invoiceAttachmentRequired=None,
                 bankFields=[],
                 purposeOfPaymentInfo=[],
                 **kwargs):

        self._validate_country_code(country)
        self._validate_currency_code(sendingCurrencies)
        self._validate_currency_code(receivingCurrencies)

        self.country = country
        self.sendingCurrencies = sendingCurrencies
        self.receivingCurrencies = receivingCurrencies
        self.purposeOfPaymentRequired = purposeOfPaymentRequired
        self.invoiceAttachmentRequired = invoiceAttachmentRequired
        self.bankFields = bankFields
        self.purposeOfPaymentInfo=[deseralize(PurposeOfPayment,
                                        p) for p in purposeOfPaymentInfo]



from veem.models.base import Base
from veem.models.exchange_rate import ExchangeRate

class ExchangeRateResponse(Base):
    def __init__(self,
                 id=None,
                 fromAmount=None,
                 fromCurrency=None,
                 fromCountry=None,
                 toAmount=None,
                 toCurrency=None,
                 toCountry=None,
                 rate=None,
                 expiry=None,
                 timeCreated=None,
                 **kwargs):

        self.id = id
        self.fromAmount = fromAmount
        self.fromCurrency = fromCurrency
        self.fromCountry = fromCountry
        self.toAmount = toAmount
        self.toCurrency = toCurrency
        self.toCountry = toCountry
        self.rate = rate
        self.expiry = expiry
        self.timeCreated = timeCreated

    @property
    def convert(self):
        return ExchangeRate(hashId=self.id, **self.json)



from veem.models.base import Base

class AddressRequest(Base):
    def __init__(self,
                 line1=None,
                 line2=None,
                 city=None,
                 stateProvince=None,
                 postalCode=None,
                 **kwargs):

        self.line1 = line1
        self.line2 = line2
        self.city = city
        self.stateProvince = stateProvince
        self.postalCode = postalCode



from attrdict import AttrDict

AccountType = AttrDict(dict(
    BUSINESS='Business',
    INCOMPLETE='Incomplete',
    PERSONAL='Personal'
))

ApprovalStatus = AttrDict(dict(
    APPROVED='Approved',
    PENDING='Pending',
    IGNORED='Ignored'
))

AttachmentType = AttrDict(dict(
    EXTERNAL_INVOICE='ExternalInvoice',
    PROOF_OF_PAYMENT='ProofOfPayment'
))

BatchItemType = AttrDict(dict(
    CONTACT='CONTACT',
    PAYMENT='PAYMENT'
))

BatchStatus = AttrDict(dict(
    FAILED='Failed',
    IN_PROGRESS='InProgress',
    COMPLETED='Completed'
))

Direction = AttrDict(dict(
    INBOUND='Inbound',
    OUTBOUND='Outbound'
))

InvoiceStatus = AttrDict(dict(
    DRAFTED='Drafted',
    SENT="Sent",
    CANCELLED="Cancelled",
    CLOSED="Closed",
    CLAIMED="Claimed",
    PAID="MarkAsPaid",
    REJECTED="Rejected"
))

PaymentSortField = AttrDict(dict(
    TIME_UPDATE='timeUpdated'
))

PaymentStatus = AttrDict(dict(
    DRAFTED="Drafted",
    SCHEDULED="Scheduled",
    PENDING_APPROVAL="PendingApproval",
    SENT="Sent",
    CLAIMED="Claimed",
    PENDING_AUTH="PendingAuth",
    AUTHORIZED="Authorized",
    IN_PROGRESS="InProgress",
    COMPLETE="Complete",
    CANCELLED="Cancelled",
    CLOSED="Closed",
    REVERSED="Reversed"
))

Scope = AttrDict(dict(
    All='all'
))

SortOrder = AttrDict(dict(
    ASC='asc',
    DESC='desc'
))

CLIENT_CREDENTIALS_GRANT_TYPE = "client_credentials"
AUTHORIZATION_CODE_GRANT_TYPE = "authorization_code"
REFRESH_TOKEN_GRANT_TYPE = "refresh_token"

FILE_DOWNLOAD_CHUNK_SIZE = 1024



from veem.models.base import Base
from veem.models.amount import Amount

from veem.utils import deseralize

class PushPaymentInfo(Base):
    def __init__(self,
                 amount=None,
                 reference=None,
                 pushPaymentInfo=None,
                 **kwargs):

        self.amount = deseralize(Amount, amount)
        self.reference = reference
        self.pushPaymentInfo = pushPaymentInfo



from veem.configuration import ConfigLoader

from veem.client.payment import PaymentClient
from veem.client.invoice import InvoiceClient
from veem.client.contact import ContactClient
from veem.client.metadata import MetadataClient
from veem.client.customer import CustomerClient
from veem.client.attachment import AttachmentClient
from veem.client.exchange_rate import ExchangeRateClient
from veem.client.authentication import AuthenticationClient

class VeemClient(object):
    """
        The Context Manager for Veem SDK Client
    """
    def __init__(self,
                 yaml_file=None,
                 configs=None,
                 useClientCredentials=False,
                 useAuthorizationCode=False,
                 **kwargs):
        self.config = ConfigLoader(yaml_file=yaml_file, configs=configs)

        self.paymentClient = PaymentClient(self.config, **kwargs)
        self.invoiceClient = InvoiceClient(self.config, **kwargs)
        self.contactClient = ContactClient(self.config, **kwargs)
        self.metadataClient = MetadataClient(self.config, **kwargs)
        self.customerClient = CustomerClient(self.config, **kwargs)
        self.attachmentClient = AttachmentClient(self.config, **kwargs)
        self.exchangeRateClient = ExchangeRateClient(self.config, **kwargs)
        self.authenticationClient = AuthenticationClient(self.config, **kwargs)

        if useClientCredentials:
            self.authenticationClient.getTokenFromClientCredentials()
        elif useAuthorizationCode:
            self.authenticationClient.getTokenFromAuthorizationCode()

    def __enter__(self):
        return self

    def __exit__(self, *args, **kwargs):
        return False



from .amount import AmountResponse

from veem.models.base import Base

from veem.utils import deseralize

class PushPaymentInfoResponse(Base):
    def __init__(self,
                 amount=None,
                 reference=None,
                 pushPaymentInfo=None,
                 **kwargs):

        self.amount = deseralize(AmountResponse, amount)
        self.reference = reference
        self.pushPaymentInfo = pushPaymentInfo




from .amount import AmountResponse
from .account import AccountResponse
from .attachment import AttachmentResponse
from .exchange_rate import ExchangeRateResponse
from .payment_approval import PaymentApprovalResponse
from .push_payment_info import PushPaymentInfoResponse

from veem.models.base import Base
from veem.models.payment import Payment

from veem.utils import deseralize

class PaymentResponse(Base):
    def __init__(self,
                 id=None,
                 requestId=None,
                 claimLink=None,
                 externalInvoiceRefId=None,
                 payee=None,
                 payer=None,
                 status=None,
                 amount=None,
                 payeeAmount=None,
                 exchangeRate=None,
                 exchangeRateResponse=None,
                 attachments=[],
                 batchItemId=None,
                 ccEmails=[],
                 exchangeRateQuoteId=None,
                 purposeOfPayment=None,
                 pushPaymentInfoResponse=None,
                 paymentApprovalResponse=None,
                 timeCreated=None,
                 timeUpdated=None,
                 timeSent=None,
                 **kwargs):

        self.id = id
        self.requestId = requestId
        self.claimLink = claimLink
        self.externalInvoiceRefId = externalInvoiceRefId
        self.payee = deseralize(AccountResponse, payee)
        self.payer = deseralize(AccountResponse, payer)
        self.status = status
        self.payeeAmount = deseralize(AmountResponse, payeeAmount)
        self.amount = deseralize(AmountResponse, amount) or self.payeeAmount
        self.exchangeRateResponse = deseralize(ExchangeRateResponse,
                                               exchangeRateResponse)
        self.exchangeRate = deseralize(ExchangeRateResponse, exchangeRate) \
                            or self.exchangeRateResponse
        self.attachments = [deseralize(AttachmentResponse,
                                       attach) for attach in attachments]
        self.batchItemId = batchItemId
        self.ccEmails = ccEmails
        self.exchangeRateQuoteId = exchangeRateQuoteId
        self.purposeOfPayment = purposeOfPayment
        self.pushPaymentInfoResponse = deseralize(PushPaymentInfoResponse,
                                                  pushPaymentInfoResponse)
        self.paymentApprovalResponse = deseralize(PaymentApprovalResponse,
                                                  paymentApprovalResponse)
        self.timeCreated = timeCreated
        self.timeUpdated = timeUpdated
        self.timeSent = timeSent

    @property
    def convert(self):
        return Payment(pushPaymentInfo=self.pushPaymentInfoResponse,
                       paymentApproval=self.paymentApprovalResponse,
                       **self.json)
